@@ -76,15 +76,8 @@ namespace librbd {
 			   << " parent completion " << m_parent_completion
 			   << " extents " << image_extents
 			   << dendl;
-    int r = aio_read(m_ictx->parent, image_extents, NULL, &m_read_data,
-	             m_parent_completion, 0);
-    if (r < 0) {
-      lderr(m_ictx->cct) << "read_from_parent " << this
-                         << ": error reading from parent: "
-                         << cpp_strerror(r) << dendl;
-      m_parent_completion->release();
-      complete(r);
-    }
+    aio_read(m_ictx->parent, image_extents, NULL, &m_read_data,
+             m_parent_completion, 0);
   }
 
   static inline bool is_copy_on_read(ImageCtx *ictx, librados::snap_t snap_id) {
@@ -238,13 +231,14 @@ namespace librbd {
     return finished;
   }
 
-  int AioRead::send() {
-    ldout(m_ictx->cct, 20) << "send " << this << " " << m_oid << " " << m_object_off << "~" << m_object_len << dendl;
+  void AioRead::send() {
+    ldout(m_ictx->cct, 20) << "send " << this << " " << m_oid << " "
+                           << m_object_off << "~" << m_object_len << dendl;
 
     // send read request to parent if the object doesn't exist locally
     if (!m_ictx->object_map.object_may_exist(m_object_no)) {
       complete(-ENOENT);
-      return 0;
+      return;
     }
 
     librados::AioCompletion *rados_completion =
@@ -261,8 +255,9 @@ namespace librbd {
     op.set_op_flags2(m_op_flags);
 
     r = m_ictx->data_ctx.aio_operate(m_oid, rados_completion, &op, flags, NULL);
+    assert(r == 0);
+
     rados_completion->release();
-    return r;
   }
 
   /** write **/
@@ -441,16 +436,13 @@ namespace librbd {
     return finished;
   }
 
-  int AbstractWrite::send() {
+  void AbstractWrite::send() {
     ldout(m_ictx->cct, 20) << "send " << this << " " << m_oid << " "
 			   << m_object_off << "~" << m_object_len << dendl;
 
-    if (send_pre()) {
-      return 0;
-    } else {
+    if (!send_pre()) {
       send_write();
     }
-    return 0;
   }
 
   bool AbstractWrite::send_pre() {
