@@ -52,8 +52,8 @@ void Elector::bump_epoch(epoch_t e)
   dout(10) << "bump_epoch " << epoch << " to " << e << dendl;
   assert(epoch <= e);
   epoch = e;
-  MonitorDBStore::Transaction t;
-  t.put(Monitor::MONITOR_NAME, "election_epoch", epoch);
+  MonitorDBStore::TransactionRef t(new MonitorDBStore::Transaction);
+  t->put(Monitor::MONITOR_NAME, "election_epoch", epoch);
   mon->store->apply_transaction(t);
 
   mon->join_election();
@@ -75,7 +75,6 @@ void Elector::start()
 
   acked_me.clear();
   classic_mons.clear();
-  required_features = mon->apply_compatset_features_to_quorum_requirements();
   init();
   
   // start by trying to elect me
@@ -214,6 +213,10 @@ void Elector::handle_propose(MMonElection *m)
   int from = m->get_source().num();
 
   assert(m->epoch % 2 == 1); // election
+  uint64_t required_features = mon->get_required_features();
+  dout(10) << __func__ << " required features " << required_features
+           << ", peer features " << m->get_connection()->get_features()
+           << dendl;
   if ((required_features ^ m->get_connection()->get_features()) &
       required_features) {
     dout(5) << " ignoring propose from mon" << from
@@ -278,6 +281,14 @@ void Elector::handle_ack(MMonElection *m)
     return;
   }
   assert(m->epoch == epoch);
+  uint64_t required_features = mon->get_required_features();
+  if ((required_features ^ m->get_connection()->get_features()) &
+      required_features) {
+    dout(5) << " ignoring ack from mon" << from
+	    << " without required features" << dendl;
+    m->put();
+    return;
+  }
   
   if (electing_me) {
     // thanks
@@ -349,7 +360,7 @@ void Elector::nak_old_peer(MMonElection *m)
   uint64_t supported_features = m->get_connection()->get_features();
 
   if (supported_features & CEPH_FEATURE_OSDMAP_ENC) {
-    uint64_t required_features = mon->apply_compatset_features_to_quorum_requirements();
+    uint64_t required_features = mon->get_required_features();
     dout(10) << "sending nak to peer " << m->get_source()
 	     << " that only supports " << supported_features
 	     << " of the required " << required_features << dendl;
@@ -358,7 +369,7 @@ void Elector::nak_old_peer(MMonElection *m)
 					   mon->monmap);
     reply->quorum_features = required_features;
     mon->features.encode(reply->sharing_bl);
-    mon->messenger->send_message(reply, m->get_connection());
+    m->get_connection()->send_message(reply);
   }
   m->put();
 }
@@ -422,9 +433,9 @@ void Elector::dispatch(Message *m)
 		<< ", taking it"
 		<< dendl;
 	mon->monmap->decode(em->monmap_bl);
-        MonitorDBStore::Transaction t;
-        t.put("monmap", mon->monmap->epoch, em->monmap_bl);
-        t.put("monmap", "last_committed", mon->monmap->epoch);
+        MonitorDBStore::TransactionRef t(new MonitorDBStore::Transaction);
+        t->put("monmap", mon->monmap->epoch, em->monmap_bl);
+        t->put("monmap", "last_committed", mon->monmap->epoch);
         mon->store->apply_transaction(t);
 	//mon->monmon()->paxos->stash_latest(mon->monmap->epoch, em->monmap_bl);
 	cancel_timer();
