@@ -116,7 +116,7 @@ int MonClient::get_monmap_privately()
   bool temp_msgr = false;
   Messenger* smessenger = NULL;
   if (!messenger) {
-    messenger = smessenger = Messenger::create(cct,
+    messenger = smessenger = Messenger::create(cct, cct->_conf->ms_type,
 					       entity_name_t::CLIENT(-1),
 					       "temp_mon_client", getpid());
     messenger->add_dispatcher_head(this);
@@ -218,7 +218,7 @@ int MonClient::ping_monitor(const string &mon_id, string *result_reply)
 
   MonClientPinger *pinger = new MonClientPinger(cct, result_reply);
 
-  Messenger *smsgr = Messenger::create(cct,
+  Messenger *smsgr = Messenger::create(cct, cct->_conf->ms_type,
 				       entity_name_t::CLIENT(-1),
 				       "temp_ping_client", getpid());
   smsgr->add_dispatcher_head(pinger);
@@ -354,7 +354,7 @@ int MonClient::init()
   Mutex::Locker l(monc_lock);
 
   string method;
-    if (cct->_conf->auth_supported.length() != 0)
+    if (!cct->_conf->auth_supported.empty())
       method = cct->_conf->auth_supported;
     else if (entity_name.get_type() == CEPH_ENTITY_TYPE_OSD ||
              entity_name.get_type() == CEPH_ENTITY_TYPE_MDS ||
@@ -372,7 +372,7 @@ int MonClient::init()
     r = keyring->from_ceph_context(cct);
     if (r == -ENOENT) {
       auth_supported->remove_supported_auth(CEPH_AUTH_CEPHX);
-      if (auth_supported->get_supported_set().size() > 0) {
+      if (!auth_supported->get_supported_set().empty()) {
 	r = 0;
 	no_keyring_disabled_cephx = true;
       } else {
