@@ -24,15 +24,17 @@
 ObjectStore *ObjectStore::create(CephContext *cct,
 				 const string& type,
 				 const string& data,
-				 const string& journal)
+				 const string& journal,
+			         osflagbits_t flags)
 {
   if (type == "filestore") {
-    return new FileStore(data, journal);
+    return new FileStore(data, journal, flags);
   }
   if (type == "memstore") {
     return new MemStore(cct, data);
   }
-  if (type == "keyvaluestore-dev") {
+  if (type == "keyvaluestore" &&
+      cct->check_experimental_feature_enabled("keyvaluestore")) {
     return new KeyValueStore(data);
   }
   return NULL;
