@@ -25,7 +25,10 @@ private:
   int set_extsize(int fd, unsigned int val);
 public:
   XfsFileStoreBackend(FileStore *fs);
-  ~XfsFileStoreBackend() {};
+  ~XfsFileStoreBackend() {}
+  const char *get_name() {
+    return "xfs";
+  }
   int detect_features();
   int set_alloc_hint(int fd, uint64_t hint);
 };
