@@ -1,6 +1,7 @@
 // -*- mode:C; tab-width:8; c-basic-offset:2; indent-tabs-mode:t -*-
 // vim: ts=8 sw=2 smarttab
 #include "test/librbd/test_fixture.h"
+#include "test/librbd/test_support.h"
 #include "include/int_types.h"
 #include "include/stringify.h"
 #include "include/rados/librados.h"
@@ -170,17 +171,17 @@ public:
     if (!ictx->image_watcher->is_lock_owner() &&
         (m_expected_aio_restarts == 0 ||
 	 m_aio_completion_restarts < m_expected_aio_restarts)) {
-      EXPECT_EQ(0, ictx->image_watcher->request_lock(
+      ictx->image_watcher->request_lock(
         boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-	aio_completion));
+	aio_completion);
     } else {
       {
 	Mutex::Locker l2(aio_completion->lock);
-	aio_completion->complete();
+	aio_completion->complete(ictx->cct);
       }
 
       m_aio_completions.erase(aio_completion);
-      delete aio_completion;
+      aio_completion->release();
     }
 
     m_callback_cond.Signal();
@@ -531,12 +532,12 @@ TEST_F(TestImageWatcher, RequestLock) {
 
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+      create_aio_completion(*ictx));
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
+      create_aio_completion(*ictx));
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -581,9 +582,9 @@ TEST_F(TestImageWatcher, RequestLockTimedOut) {
 
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
+      create_aio_completion(*ictx));
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -608,9 +609,9 @@ TEST_F(TestImageWatcher, RequestLockTryLockRace) {
 
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
+      create_aio_completion(*ictx));
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
@@ -643,9 +644,9 @@ TEST_F(TestImageWatcher, RequestLockPreTryLockFailed) {
 
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
+      create_aio_completion(*ictx));
   }
   ASSERT_TRUE(wait_for_aio_completions(*ictx));
 }
@@ -665,9 +666,9 @@ TEST_F(TestImageWatcher, RequestLockPostTryLockFailed) {
   m_expected_aio_restarts = 1;
   {
     RWLock::WLocker l(ictx->owner_lock);
-    ASSERT_EQ(0, ictx->image_watcher->request_lock(
+    ictx->image_watcher->request_lock(
       boost::bind(&TestImageWatcher::handle_restart_aio, this, ictx, _1),
-      create_aio_completion(*ictx)));
+      create_aio_completion(*ictx));
   }
 
   ASSERT_TRUE(wait_for_notifies(*ictx));
